/* Copyright (c) 2014 Petr Bilek
   Tento kod je volne programove vybaveni; muzete jej sirit a modifikovat
podle ustanoveni Obecne verejne license GNU, vydavane FreeSoftware Foundation.
Na tento program se nevztahuji ZADNE ZARUKY; neposkytuji se ani odvozene zaruky
PRODEJNSTI anebo VHODNOSTI PRO URCITY UCEL. */
/* 51ncurses/thread-ncurses.c                     */
#include "thread-ncurses.h"
#include <pthread.h>
#include <semaphore.h>

#define MAX_JOBS 10
#define MAX_WINDOWS 10

static THNcursesJob jobs[MAX_JOBS];
static int jobs_index = 0;
static WINDOW *windows[MAX_WINDOWS];
static int windows_index = 0;
static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
static sem_t sem;

void *
thncurses_signal_thread (void *tjob)
{
  int i;
  THNcursesJob *job = (THNcursesJob *) tjob;

  i = pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
  if (i)
    {
      fprintf (stderr,
	       "thncurses_signal_thread: Selhalo nastaveni priznaku ukonceni funkce varovani");
      return NULL;
    }

  sem_init (&sem, 0, 0);

  while (true)
    {
      while (sem_wait (&sem) != 0)
	{
	  pthread_testcancel ();
	}
      pthread_testcancel ();
      thncurses_add_job (*job);
      thncurses_refresh_windows ();
    }
}

void
thncurses_signal (int signal)
{
  sem_post (&sem);
}

bool
thncurses_add_job (THNcursesJob job)
{
  while (pthread_mutex_lock (&mutex));
  if (jobs_index == MAX_JOBS)
    {
      pthread_mutex_unlock (&mutex);
      return false;
    }
  jobs[jobs_index++] = job;
  pthread_mutex_unlock (&mutex);
  return true;
}

bool
thncurses_add_window (WINDOW * win)
{
  while (pthread_mutex_lock (&mutex));
  if (windows_index == MAX_WINDOWS)
    {
      pthread_mutex_unlock (&mutex);
      return false;
    }
  windows[windows_index++] = win;
  pthread_mutex_unlock (&mutex);
  return true;
}

void
thncurses_refresh_windows (void)
{
  while (pthread_mutex_lock (&mutex));
  int j;
  for (j = 0; j < jobs_index; j++)
    {
      jobs[j].fce (jobs[j].data);
    }
  jobs_index = 0;
  j = 0;
  while (windows[j])
    {
      wnoutrefresh (windows[j++]);
    }
  doupdate ();
  pthread_mutex_unlock (&mutex);
}
